@@ -91,7 +91,7 @@ sub new ($$$) {
     $self->{init_count} = 5;
 
     my $table = "interfaces.ifTable.ifEntry";
-    my $num = $self->{num};
+    my $num = $self->{'num'};
     $self->{vars} = [ SNMP::Varbind->new(["$table.ifDescr", $num]),
 		      SNMP::Varbind->new(["$table.ifInOctets", $num]),
 		      SNMP::Varbind->new(["$table.ifOutOctets", $num]),
@@ -99,6 +99,14 @@ sub new ($$$) {
 		      SNMP::Varbind->new(["$table.ifType", $num]),
 		      SNMP::Varbind->new(["$table.ifAdminStatus", $num]),
 		      SNMP::Varbind->new(["$table.ifOperStatus", $num]) ];
+    if ($self->{'combo'}) {
+	foreach my $num (@{$self->{'combo'}}) {
+	    push(@{$self->{'vars'}},
+		 SNMP::Varbind->new(["$table.ifInOctets", $num]),
+		 SNMP::Varbind->new(["$table.ifOutOctets", $num]),
+		 SNMP::Varbind->new(["$table.ifSpeed", $num]));
+	}
+    }
 
     #
     #   Decide whether this is a full duplex interface; code borrowed
@@ -129,47 +137,61 @@ sub new ($$$) {
 }
 
 
-sub Process ($) {
-    my($self) = @_;
-    my $session = $self->{session};
-    my $dbh = $session->{config}->{dbh};
-    my $vr_session = $session->{vars_registered};
-    my $vr_self = $self->{vars_registered};
+sub GetVars ($) {
+    my $self = shift;
+    my $session = $self->{'session'};
+    my $vr_session = $session->{'vars_registered'};
+    my $vr_self = $self->{'vars_registered'};
 
     # The following list corresponds to the list in the 'new' method.
-    # This is important when calculation the index $i in $vr_self->[$i].
-    my $ifDescr = $vr_session->[$vr_self->[0]]->[0]->[2];
-    my $ifInOctets = $vr_session->[$vr_self->[1]]->[0]->[2];
-    my $ifOutOctets = $vr_session->[$vr_self->[2]]->[0]->[2];
-    my $ifSpeed = $vr_session->[$vr_self->[3]]->[0]->[2];
-    my $ifType = $vr_session->[$vr_self->[4]]->[0]->[2];
-    my $ifAdminStatus = $vr_session->[$vr_self->[5]]->[0]->[2];
-    my $ifOperStatus = $vr_session->[$vr_self->[6]]->[0]->[2];
-    my $num = $self->{num};
+    # This is important when calculating the index $i in $vr_self->[$i].
+    my %v;
+    my $i = 0;
+    $v{'ifDescr'}       = $vr_session->[$vr_self->[$i++]]->[0]->[2];
+    $v{'ifInOctets'}    = $vr_session->[$vr_self->[$i++]]->[0]->[2];
+    $v{'ifOutOctets'}   = $vr_session->[$vr_self->[$i++]]->[0]->[2];
+    $v{'ifSpeed'}       = $vr_session->[$vr_self->[$i++]]->[0]->[2];
+    $v{'ifType'}        = $vr_session->[$vr_self->[$i++]]->[0]->[2];
+    $v{'ifAdminStatus'} = $vr_session->[$vr_self->[$i++]]->[0]->[2];
+    $v{'ifOperStatus'}  = $vr_session->[$vr_self->[$i++]]->[0]->[2];
 
-    if ($self->{description} ne $ifDescr  ||
-	$self->{speed} ne $ifSpeed        ||
-	$self->{type} ne $ifType) {
-	if (!$self->{err_msg_mismatch}) {
-	    my $cDescr = $self->{description};
-	    my $cSpeed = $self->{speed};
-	    my $cType = $self->{type};
-	    $self->{err_msg_mismatch} =
+    if ($self->{'combo'}) {
+	$v{'ifUtilSpeed'} = $v{'ifSpeed'};
+	foreach my $num (@{$self->{'combo'}}) {
+	    $v{'ifInOctets'} += $vr_session->[$vr_self->[$i++]]->[0]->[2];
+	    $v{'ifOutOctets'} += $vr_session->[$vr_self->[$i++]]->[0]->[2];
+	    $v{'ifUtilSpeed'} += $vr_session->[$vr_self->[$i++]]->[0]->[2];
+	}
+    }
+
+    return wantarray ? (\%v, $i) : \%v;
+}
+
+
+sub Verify ($$) {
+    my $self = shift; my $v = shift;
+
+    my $num = $self->{'num'};
+    if ($self->{'description'} ne $v->{'ifDescr'}  ||
+	$self->{'speed'}       ne $v->{'ifSpeed'}  ||
+	$self->{'type'}        ne $v->{'ifType'}) {
+	if (!$self->{'err_msg_mismatch'}) {
+	    $self->{'err_msg_mismatch'} =
 		$self->Message(subject => 'Router config mismatch',
 			       body => <<"MSG");
 
 The configuration of interface $num doesn't match the detected parameters.
 The configured parameters are:
 
-    Interface description:  $cDescr
-    Interface speed:        $cSpeed
-    Interface type:         $cType
+    Interface description:  $self->{'description'}
+    Interface speed:        $self->{'speed'}
+    Interface type:         $self->{'type'}
 
 The detected parameters are:
 
-    Interface description:  $ifDescr
-    Interface speed:        $ifSpeed
-    Interface type:         $ifType
+    Interface description:  $v->{'ifDescr'}
+    Interface speed:        $v->{'ifSpeed'}
+    Interface type:         $v->{'ifType'}
 
 I won't send further messages until the configured parameters match or
 the SNMP::Monitor is restarted.
@@ -177,24 +199,44 @@ the SNMP::Monitor is restarted.
 MSG
         }
     } else {
-	$self->{err_msg_mismatch} = 0;
-    }
+	if ($self->{'err_msg_mismatch'}) {
+	    $self->{err_msg_mismatch} =
+		!$self->Message(subject => 'Router config mismatch is gone',
+				body => <<"MSG");
 
+The configuration of interface $num didn't match the detected parameters.
+This seems to be the case no longer. The detected parameters are:
+
+    Interface description:  $self->{'description'}
+    Interface speed:        $self->{'speed'}
+    Interface type:         $self->{'type'}
+
+MSG
+	}
+    }
+}
+
+
+sub Calculate ($$) {
+    my $self = shift; my $v = shift;
+    my $session = $self->{'session'};
     my $time = $session->{'time'};
     my $oldTime = $self->{'time'};
     $self->{'time'} = $time;
 
-    my $oldIfInOctets = $self->{ifInOctets};
-    $self->{ifInOctets} = $ifInOctets;
-    if (defined($oldIfInOctets)  &&  $ifInOctets > $oldIfInOctets) {
+    my $ifInOctets = $v->{'ifInOctets'};
+    my $oldIfInOctets = $self->{'ifInOctets'};
+    $self->{'ifInOctets'} = $ifInOctets;
+    if (defined($oldIfInOctets)) {
 	$ifInOctets -= $oldIfInOctets;
     } else {
 	$ifInOctets = 0;
     }
 
-    my $oldIfOutOctets = $self->{ifOutOctets};
-    $self->{ifOutOctets} = $ifOutOctets;
-    if (defined($oldIfOutOctets)  &&  $ifOutOctets > $oldIfOutOctets) {
+    my $ifOutOctets = $v->{'ifOutOctets'};
+    my $oldIfOutOctets = $self->{'ifOutOctets'};
+    $self->{'ifOutOctets'} = $ifOutOctets;
+    if (defined($oldIfOutOctets)) {
 	$ifOutOctets -= $oldIfOutOctets;
     } else {
 	$ifOutOctets = 0;
@@ -203,39 +245,73 @@ MSG
     my $utilization;
     if ($ifInOctets > 0  ||  $ifOutOctets > 0) {
 	my $delta;
-	if ($self->{full_duplex}) {
+		if ($self->{'full_duplex'}) {
 	    $delta = ($ifInOctets > $ifOutOctets) ? $ifInOctets : $ifOutOctets;
 	} else {
 	    $delta = $ifInOctets + $ifOutOctets;
 	}
-	$utilization = ($delta * 8 * 100.0) / (($time - $oldTime) * $ifSpeed);
+	$utilization = ($delta * 8 * 100.0) /
+	               (($time - $oldTime) * 
+			($v->{'ifUtilSpeed'} || $v->{'ifSpeed'}));
     } else {
 	$utilization = 0.0;
     }
 
-    if ($session->{config}->{debug}) {
-        Sys::Syslog::syslog('debug',
-			    "IfLoad: Host %s, interface %d: InOctets %s => %d,"
-			    . " OutOctets %s => %d",
-			    $session->{name}, $self->{num},
-			    defined($oldIfInOctets) ? $oldIfInOctets : "undef",
-			    $ifInOctets,
-			    defined($oldIfOutOctets) ?
-			        $oldIfOutOctets : "undef",
-			    $ifOutOctets);
+    $v->{'ifInOctets'} = $ifInOctets;
+    $v->{'oldIfInOctets'} = $oldIfInOctets;
+    $v->{'ifOutOctets'} = $ifOutOctets;
+    $v->{'oldIfOutOctets'} = $oldIfOutOctets;
+    $v->{'utilization'} = $utilization;
+}
+
+
+sub _FromUnixTime {
+    my($sec, $min, $hour, $mday, $mon, $year) = localtime(shift);
+    sprintf('%04d-%02d-%02d %02d:%02d:%02d',
+	    $year+1900, $mon+1, $mday, $hour, $min, $sec);
+}
+
+sub Query {
+    my($self, $session, $v) = @_;
+    my $dbh = $session->{config}->{dbh};
+
+    my $name = $session->{'name'};
+    my $num = $self->{'num'};
+    my $ifInOctets = $v->{'ifInOctets'} || 0;
+    my $oldIfInOctets = $v->{'oldIfInOctets'};
+    my $ifOutOctets = $v->{'ifOutOctets'} || 0;
+    my $oldIfOutOctets = $v->{'oldIfOutOctets'};
+
+    if ($session->{'config'}->{'debug'}) {
+        Sys::Syslog::syslog
+	    ('debug',
+	     "IfLoad: Host %s, interface %d: InOctets %s => %d,"
+	     . " OutOctets %s => %d",
+	     $name, $num,
+	     defined($oldIfInOctets) ? $oldIfInOctets : "undef",
+	     $ifInOctets, defined($oldIfOutOctets) ? $oldIfOutOctets : "undef",
+	     $ifOutOctets);
     }
 
-    if (!$dbh->do("INSERT INTO SNMPMON_IFLOAD VALUES (?, ?, FROM_UNIXTIME(?),"
-		  . " ?, ?, ?, ?, ?)",
-		  undef, $session->{name}, $num,
-		  $session->{'time'},
-		  $ifInOctets || 0,
-		  $ifOutOctets || 0,
-		  $utilization || 0.0,
-		  ($ifAdminStatus || 0),
-		  ($ifOperStatus || 0))) {
+    $dbh->do("INSERT INTO SNMPMON_IFLOAD VALUES (?, ?, ?,"
+	     . " ?, ?, ?, ?, ?)",
+	     undef, $name, $num, _FromUnixTime($session->{'time'}),
+	     $ifInOctets,
+	     $ifOutOctets,
+	     $v->{'utilization'} || 0.0,
+	     ($v->{'ifAdminStatus'} || 0),
+	     ($v->{'ifOperStatus'} || 0));
+}
+
+
+sub Log {
+    my $self = shift; my $v = shift;
+    my $session = $self->{session};
+
+    if (!$self->Query($session, $v)) {
+	my $dbh = $session->{'config'}->{'dbh'};
 	my $errmsg = $dbh->errstr();
-	my $host = $session->{name};
+	my $host = $session->{'name'};
 
 	$self->Message(subject => 'Database error',
 		       body => <<"MSG")
@@ -243,13 +319,13 @@ MSG
 A database error occurred, while logging the following values:
 
     Host name:         $host
-    Interface number:  $num
-    Time:              $time
-    InOctets:          $ifInOctets
-    OutOctets:         $ifOutOctets
-    Utilization:       $utilization
-    AdminStatus:       $ifAdminStatus
-    OperStatus:        $ifOperStatus
+    Interface number:  $self->{'num'}
+    Time:              $session->{'time'}
+    InOctets:          $v->{'ifInOctets'}
+    OutOctets:         $v->{'ifOutOctets'}
+    Utilization:       $v->{'utilization'}
+    AdminStatus:       $v->{'ifAdminStatus'}
+    OperStatus:        $v->{'ifOperStatus'}
 
 The database error message was:
 
@@ -263,4 +339,15 @@ MSG
 }
 
 
+sub Process ($) {
+    my $self = shift;
+    my $session = $self->{session};
+    my $v = $self->GetVars();
+
+    $self->Verify($v);
+    $self->Calculate($v);
+    $self->Log($v);
+}
+
+
 1;
